package com.netpotlet.test;

import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;
import java.util.Date;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;

import org.exolab.castor.jdo.Database;
import org.exolab.castor.jdo.DatabaseNotFoundException;
import org.exolab.castor.jdo.JDO;
import org.exolab.castor.jdo.OQLQuery;
import org.exolab.castor.jdo.PersistenceException;
import org.exolab.castor.jdo.QueryResults;
import org.exolab.castor.mapping.MappingException;
import org.exolab.castor.xml.MarshalException;
import org.exolab.castor.xml.Marshaller;
import org.exolab.castor.xml.ValidationException;

import com.netpotlet.nymph.*;

public class CastorJDOTest {
    private CastorJDOTest(String xmlfile, String conffile,
                          String xsltfile, String outputfile)
        throws MappingException,
               DatabaseNotFoundException,
               PersistenceException,
               MarshalException,
               ValidationException,
               TransformerConfigurationException,
               IOException,
               FactoryConfigurationError,
               ParserConfigurationException, TransformerException {

        GoodsList goodsList = unmarshall(xmlfile);
        JDO jdo = new JDO();
        jdo.setConfiguration(getClass().getResource(conffile).toString());
        jdo.setDatabaseName("nymph");
        Database database = jdo.getDatabase();
        create(database, goodsList);
        addData(database);
        GoodsList selectedList = select(database);
        String marshalledString = getMarshalledString(selectedList);
        transform(xsltfile, marshalledString, outputfile);
    }
    
    private GoodsList unmarshall(String filename)
        throws MarshalException,
               ValidationException,
               FileNotFoundException {
        
        GoodsList goodsList =
            (GoodsList)GoodsList.unmarshal(new FileReader(filename));
        return goodsList;
    }

    private void create(Database database, GoodsList goodsList)
        throws PersistenceException {
        Goods[] goodsArray = goodsList.getGoods();
        for (int i=0; i<goodsArray.length; i++) {
            JdoGoods jdoGoods = new JdoGoods(goodsArray[i]);
            create(database, jdoGoods);
        }
    }

    private void create(Database database, JdoGoods goods)
        throws PersistenceException {
        Price price = goods.getPrice();
        Category category = goods.getCategory();
        database.begin();
        database.create(goods);
        database.create(price);
        database.create(category);
        database.commit();
    }
    
    private void addData(Database database)
        throws PersistenceException {
        int id = 5;
        JdoGoods goods = new JdoGoods();
        goods.setId(5);
        goods.setSample("river.gif");
        goods.setName("եȥɣ");
        goods.setDate(2003, 12-1, 12);
        goods.setDescription("");
        Price price = new Price();
        price.setId(id);
        price.setCost(100);
        price.setRetail(150);
        goods.setPrice(price);
        Category category = new Category();
        category.setId(id);
        category.setName("ʸ");
        goods.setCategory(category);
        create(database, goods);
    }
    
    private GoodsList select(Database database)
        throws PersistenceException {
        GoodsList selectedList = new GoodsList();
        database.begin();
        OQLQuery oqlQuery =
          database.getOQLQuery("SELECT goods FROM com.netpotlet.nymph.JdoGoods goods");
        QueryResults results = oqlQuery.execute();
        while (results.hasMoreElements()) {
            JdoGoods jdoGoods = (JdoGoods)results.next();
            int id = jdoGoods.getId();
            String sample = (String)jdoGoods.getSample();
            String name = jdoGoods.getName();
            Date date = jdoGoods.getDate();
            String description = jdoGoods.getDescription();
            Price price = jdoGoods.getPrice();
            Category category = jdoGoods.getCategory();
            System.out.println(id+", "+sample+", "+name+", "+description);
            System.out.println(price.getCost()+", "+price.getRetail());
            System.out.println(category.getName());
            selectedList.addGoods(jdoGoods.getGoods());
        }
        results.close();
        oqlQuery.close();
        database.commit();
        return selectedList;
    }
    
    private String getMarshalledString(GoodsList goodsList)
        throws IOException,
               MarshalException,
               ValidationException {
        StringWriter writer = new StringWriter();
        Marshaller marshaller = new Marshaller(writer);
        marshaller.setEncoding("euc-jp");
        marshaller.marshal(goodsList);
        return writer.toString();
    }
    
    private void transform(String xsltfile, String document, String outputfile)
        throws TransformerConfigurationException,
               IOException,
               FactoryConfigurationError,
               ParserConfigurationException,
               TransformerException {
        DocumentTransformer transformer = 
            new DocumentTransformer("euc-jp", xsltfile);
        transformer.setSource(document);
        transformer.setResult(outputfile);
        transformer.transformDocument();
    }

    public static void main(String[] args) throws Exception {
        new CastorJDOTest(args[0], args[1], args[2], args[3]);
    }
}
