package com.netpotlet.test;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.StringReader;
import java.io.UnsupportedEncodingException;
import java.io.IOException;
import java.net.URL;

import javax.xml.parsers.FactoryConfigurationError;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.stream.StreamResult;
import javax.xml.transform.stream.StreamSource;

class DocumentTransformer {
    private String encoding;
    private TransformerFactory factory;
    private String xsltfile;
    private Source source = null;
    private Result result = null;

    DocumentTransformer(String encoding, String xsltfile)
        throws IOException {
        this.encoding = encoding;
        factory = TransformerFactory.newInstance();
        this.xsltfile = xsltfile;
    }

    void transformDocument()
        throws
            IOException,
            FactoryConfigurationError,
            ParserConfigurationException,
            TransformerConfigurationException,
            TransformerException {
        URL url = new File(xsltfile).toURL();
        Source xsltSource = new StreamSource(url.toExternalForm());
        Transformer transformer = factory.newTransformer(xsltSource);
        if ((source != null) && (result != null)) {
            transform(transformer);
        }
    }

    void setSource(String document) {
        StringReader reader = new StringReader(document);
        source = new StreamSource(reader);
    }
    
    void setResult(String filename)
        throws UnsupportedEncodingException,
               FileNotFoundException {
        OutputStreamWriter writer = 
            new OutputStreamWriter(new FileOutputStream(filename),
                                   encoding);
        result = new StreamResult(writer);
    }

    private void transform(Transformer transformer)
        throws TransformerException {
        transformer.setOutputProperty(OutputKeys.ENCODING, encoding);
        transformer.setOutputProperty(OutputKeys.INDENT, "yes");
        transformer.transform(source, result);
    }
}
